#include <stdio.h>
#include "date.h"

#define TZONE          -6  /* CST */
#define OUTPUT_FORMAT  DT_US_FORMAT

int main(void)
{
  DT_DATE *date;
  DT_EVENT *event;
  int inc;
  int y,m,d;

/*   printf("YYYY/MM/DD: "); */
/*   scanf("%d/%d/%d", &y, &m, &d); */

  /* 2000 Jan 2 */
  y = 2000;
  m = 1;
  d = 2;

  date = DT_mkdate(y,m,d);
  printf("\nUS: %s\n",
	 DT_ascii(date, DT_US_FORMAT));
  printf("US Numeric: %s\n",
	 DT_ascii(date, DT_US_FORMAT|DT_NUMERIC_OUT));
  printf("Europe: %s\n",
	 DT_ascii(date, DT_EUROPE_FORMAT));
  printf("Europe Numeric: %s\n",
	 DT_ascii(date, DT_EUROPE_FORMAT|DT_NUMERIC_OUT));
  printf("Universal: %s\n",
	 DT_ascii(date, DT_UNIVERSAL_FORMAT));
  printf("Universal Numeric: %s\n\n",
	 DT_ascii(date, DT_UNIVERSAL_FORMAT|DT_NUMERIC_OUT));

  printf("rata die: %ld\n", DT_rata_die(date));
  printf("moon age: %g\n", DT_moon_age(date, TZONE));
  printf("leap year? %s\n\n", DT_is_leap_year(date)?"Yes":"No");
  printf("Days this month: %d\n", DT_days_this_month(date));


  /* Test DT_add_days */
  for (inc = 1; inc <= 2; inc++)
    {
      printf("(+%d %s)\n", inc, (inc==1?"day":"days"));
      DT_add_days(date, 1);
      
      printf("date: %s\n", DT_ascii(date, DT_US_FORMAT));
      printf("moon age: %g\n\n", DT_moon_age(date, TZONE));
    }

  /* Test DT_add_months */
  for (inc = 1; inc <= 2; inc++)
    {
      printf("(+%d %s)\n", inc, (inc==1?"month":"months"));
      DT_add_months(date, 1);
      
      printf("date: %s\n", DT_ascii(date, DT_US_FORMAT));
      printf("moon age: %g\n\n", DT_moon_age(date, TZONE));
    }

  DT_find_dow_date(date, DT_THIRD, DT_MONDAY, DT_JANUARY);
  printf("Martin Luther King's Birthday : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_THIRD, DT_MONDAY, DT_FEBRUARY);
  printf("              Presidents' Day : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_FIRST, DT_SUNDAY, DT_APRIL);
  printf(" Daylight Savings Time Begins : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_easter(date);
  printf("                       Easter : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_LAST, DT_MONDAY, DT_MAY);
  printf("                 Memorial Day : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_FIRST, DT_MONDAY, DT_SEPTEMBER);
  printf("                    Labor Day : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_SECOND, DT_MONDAY, DT_OCTOBER);
  printf("                 Columbus Day : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_LAST, DT_SUNDAY, DT_OCTOBER);
  printf("   Daylight Savings Time Ends : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_FOURTH, DT_THURSDAY, DT_NOVEMBER);
  printf("                 Thanksgiving : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  DT_find_dow_date(date, DT_LAST, DT_DAY, DT_DECEMBER);
  printf("         Last day of December : %s\n",
	 DT_ascii(date, OUTPUT_FORMAT));

  /***** Event handling *****/
  /* Create an event that starts on the third tuesday in august 2000, ends
   * on the second Thursday in december 200, and occurs on tuesdays and
   * thursdays of every 2nd week */
/*   event = DT_mkevent(DT_mkdate_dow(DT_THIRD, DT_TUESDAY, */
/* 				   DT_AUGUST, 2000), */
/* 		     DT_mkdate_dow(DT_SECOND, DT_THURSDAY, */
/* 				   DT_DECEMBER, 2000), 14); */

  event = DT_mkevent_monthly(DT_mkdate_dow(DT_THIRD, DT_TUESDAY,
					   DT_AUGUST, 2000),
			     DT_mkdate_dow(DT_SECOND, DT_THURSDAY,
					   DT_DECEMBER, 2002), 1, 3);

/*   DT_event_adj_rel(event, 3); */
  DT_event_adj_dow(event, DT_PREV, DT_SUNDAY);
/*   DT_event_pattern(event, "101"); */
  DT_set_rata_die(date, DT_rata_die(event->start_date));
  printf("\nOccurances of our event (DT_is_event_today)\n");

  /* Understand, of course, that this is a terrbible way of finding all
   * occurances of an event for a given period of time.  Use DT_event_next
   * instead (see below). */
  while (DT_rata_die(date) <= DT_rata_die(event->end_date))
    {
      if (DT_is_event_today(date, event))
	{
	  printf("%9s, %18s\n", DT_dow_ascii(date), 
		 DT_ascii(date, DT_US_FORMAT));
	}
      
      DT_add_days(date, 1);
    }

  printf("\nOccurances of the same event (DT_event_next)\n");
  DT_set_rata_die(date, DT_rata_die(event->start_date)-1);

  /* Now, do it the right way */
  while (DT_event_next(date, event))
    {
      printf("%9s, %18s\n", DT_dow_ascii(date), 
	     DT_ascii(date, DT_US_FORMAT));
    }

  DT_rmevent(event, DT_FREE_ALL);
  DT_rmdate(date);
  return 0;
}
